<?php

namespace Sale\Handlers\PaySystem;

use Bitrix\Main\Localization;
use Bitrix\Main;
use Bitrix\Main\Request;
use Bitrix\Main\Web\HttpClient;
use Bitrix\Sale\Payment;
use Bitrix\Sale\PaymentCollection;
use Bitrix\Sale\PaySystem;
use Bitrix\Sale\PriceMaths;

Localization\Loc::loadMessages(__FILE__);


class PassimpayHandler
extends PaySystem\ServiceHandler
implements PaySystem\IRefund 
{
    const PAYMENT_TYPE="PAYMENT";
    const PAYED_STATUS="PROCESSED";



	public function initiatePay(Payment $payment, Request $request = null)
	{




        $url = 'https://api.passimpay.io/createorder';
        $platform_id = $this->getBusinessValue($payment, 'PLATFORM_ID'); // Platform ID
        $apikey = $this->getBusinessValue($payment, 'SECRET_KEY'); // Secret key
        $usdRate = $this->getBusinessValue($payment, 'USD_RATE'); // Secret key
        $order_id = $payment->getField("ORDER_ID"); // Payment ID of your platform
        $amount =  number_format($payment->getSum(),"2",".","");     // USD, decimals - 2
        if($usdRate){
            $amount =  number_format($amount/$usdRate,"2",".","");
        }
        //$this->getCurList($platform_id,$apikey);

        $payload = http_build_query(['platform_id' => $platform_id, 'order_id' => $order_id, 'amount' => $amount]);
        $hash = hash_hmac('sha256', $payload, $apikey);

        $data = [
            'platform_id' => $platform_id,
            'order_id' => $order_id,
            'amount' => $amount,
            'hash' => $hash
        ];

        $post_data = http_build_query($data);

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_HEADER, false);
        curl_setopt($curl, CURLOPT_HTTPHEADER, ['Content-Type: application/x-www-form-urlencoded']);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $post_data);
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_ENCODING, 'gzip');
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
        $result = curl_exec($curl);
        curl_close( $curl );

        $result = json_decode($result, true);


        if (isset($result['result']) && $result['result'] == 1)
        {
            $url = $result['url'];

            ?>
            <script>
                setTimeout('location.href="<?=$url?>";',500);
            </script>
            You will be redirected to the payment page. If this did not happen, click on <a href="<?=$url?>">go to payment</a><br><br>
            <?

        }

        else
        {
            $error = $result['message']; // Error text

            echo "[".$error."] ";
            echo "<br>";
            $result = new PaySystem\ServiceResult();
            $result->addError(new Main\Error("[".$error."] "));
            return $result;
        }




        $result = new PaySystem\ServiceResult();
        return $result;



	}
	
	public function confirm(Payment $payment, $sum = 0)
	{
		
	}
	
	public function cancel(Payment $payment)
	{
		
	}
	
	public function refund(Payment $payment, $refundableSum)
	{
		
		
	}

    public static function isMyResponse(Request $request, $paySystemId)
    {

        file_put_contents($_SERVER["DOCUMENT_ROOT"]."/pay.php","\n".Date("H:i:s: ")." requested ============================================================================",FILE_APPEND);
        file_put_contents($_SERVER["DOCUMENT_ROOT"]."/pay.php","\n".Date("H:i:s: ").print_r($_POST,1),FILE_APPEND);
        if (isset($_POST['txhash'])&&isset($_POST['platform_id'])&&isset($_POST['amount']))
        {
            return true;
        }
        return false;
    }



	public function processRequest(Payment $payment, Request $request)
	{



        file_put_contents($_SERVER["DOCUMENT_ROOT"]."/pay.php","\n".Date("H:i:s: ")."process",FILE_APPEND);
        $result = new PaySystem\ServiceResult();
        $platform_id = $this->getBusinessValue($payment, 'PLATFORM_ID'); // Platform ID
        $apikey = $this->getBusinessValue($payment, 'SECRET_KEY'); // Secret key
        $usdRate = $this->getBusinessValue($payment, 'USD_RATE'); // Secret key
        $order_id = $payment->getField("ORDER_ID"); // Payment ID of your platform
        $amount =  number_format($payment->getSum(),"2",".","");     // USD, decimals - 2
        file_put_contents($_SERVER["DOCUMENT_ROOT"]."/pay.php","\n".Date("H:i:s: ")."found  amount ".$amount,FILE_APPEND);
        if($usdRate){
            $amount =  number_format($amount/$usdRate,"2",".","");
        }


        $hash = $_POST['hash'];
        $data = [
            'platform_id' => (int) $_POST['platform_id'], // Platform ID
            'payment_id' => (int) $_POST['payment_id'], // currency ID
            'order_id' => (int) $_POST['order_id'], // Payment ID of your platform
            'amount' => $_POST['amount'], // transaction amount
            'txhash' => $_POST['txhash'], // transaction ID in the cryptocurrency network
            'address_from' => $_POST['address_from'], // sender address
            'address_to' => $_POST['address_to'], // recipient address
            'fee' => $_POST['fee'], // network fee
        ];
        if (isset($_POST['confirmations'])) $data['confirmations'] = $_POST['confirmations'];
        $payload = http_build_query($data);
        if (!isset($hash) || hash_hmac('sha256', $payload, $apikey) != $hash) {

        } else {
            file_put_contents($_SERVER["DOCUMENT_ROOT"]."/pay.php","\n".Date("H:i:s: ")."valid",FILE_APPEND);
            $orderStatus=$this->getOrderStatus($platform_id,$apikey,(int) $_POST['order_id']);

            if($orderStatus=="paid"){
                $fields = array(
                    "PS_STATUS_CODE" => $transaction['txhash'],
                    "PS_STATUS_DESCRIPTION" => "from ".$_POST['address_from']  ." to ".$_POST['address_to'] ,
                    "PS_STATUS_MESSAGE" => $transaction['id'],
                    "PS_INVOICE_ID" => $_POST['txhash'],
                    "PS_SUM" =>$_POST['amount'],
                    "PS_STATUS" => 'N',
                    "PS_CURRENCY" =>$transaction['currency'],
                    "PS_RESPONSE_DATE" => new Main\Type\DateTime()
                );
                $fields["PS_STATUS"] = 'Y';
                $payment->setPaid("Y");
                $result->setPsData($fields);
            }


            /*

            $fields = array(
                "PS_STATUS_CODE" => $transaction['txhash'],
                "PS_STATUS_DESCRIPTION" => "from ".$_POST['address_from']  ." to ".$_POST['address_to'] ,
                "PS_STATUS_MESSAGE" => $transaction['id'],
                "PS_INVOICE_ID" => $_POST['txhash'],
                "PS_SUM" =>$_POST['amount'],
                "PS_STATUS" => 'N',
                "PS_CURRENCY" =>$transaction['currency'],
                "PS_RESPONSE_DATE" => new Main\Type\DateTime()
            );

            file_put_contents($_SERVER["DOCUMENT_ROOT"]."/pay.php","\n".Date("H:i:s: ")."calc amount ".$amount,FILE_APPEND);
            if( $amount > $_REQUEST['amount'] ) {
                $error = "Сумма платежа не совпадает";
                file_put_contents($_SERVER["DOCUMENT_ROOT"]."/pay.php","\n".Date("H:i:s: ").$error,FILE_APPEND);
                $fields['PS_STATUS_DESCRIPTION'] .= ' '.$error;
                $result->addError(new Main\Error($error));
            } else {
                $fields["PS_STATUS"] = 'Y';
                $payment->setPaid("Y");
                file_put_contents($_SERVER["DOCUMENT_ROOT"]."/pay.php","\n".Date("H:i:s: ")." SET PAID",FILE_APPEND);
                if ($this->getBusinessValue($payment, 'CHANGE_STATUS_PAY') === 'Y')
                {
                    file_put_contents($_SERVER["DOCUMENT_ROOT"]."/pay.php","\n".Date("H:i:s: ")." CHANGE_STATUS_PAY",FILE_APPEND);
                    $result->setOperationType(PaySystem\ServiceResult::MONEY_COMING);
                }
            }

            $result->setPsData($fields);*/
        }



        echo "OK";

        return $result;
	}


    private function getOrderStatus($platform_id,$apikey,$orderId){
        file_put_contents($_SERVER["DOCUMENT_ROOT"]."/pay.php","\n".Date("H:i:s: ")." getOrderStatus",FILE_APPEND);
        $url = 'https://api.passimpay.io/orderstatus';
        $payload = http_build_query(['platform_id' => $platform_id, 'order_id' => $orderId ]);
        $hash = hash_hmac('sha256', $payload, $apikey);
        $data = [
            'platform_id' => $platform_id,
            'order_id' => $orderId,
            'hash' => $hash
        ];
        $post_data = http_build_query($data);
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_HEADER, false);
        curl_setopt($curl, CURLOPT_HTTPHEADER, ['Content-Type: application/x-www-form-urlencoded']);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $post_data);
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_ENCODING, 'gzip');
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
        $result = curl_exec($curl);
        curl_close( $curl );
        file_put_contents($_SERVER["DOCUMENT_ROOT"]."/pay.php","\n".Date("H:i:s: ")." result= ".$result,FILE_APPEND);
        $result = json_decode($result, true);
        return $result['status'];

    }

    private function getCurList($platform_id,$apikey)
    {
        $url = 'https://api.passimpay.io/currencies';
        $payload = http_build_query(['platform_id' => $platform_id ]);
        $hash = hash_hmac('sha256', $payload, $apikey);
        $data = [ 'platform_id' => $platform_id, 'hash' => $hash,];
        $post_data = http_build_query($data);
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_HEADER, false);
        curl_setopt($curl, CURLOPT_HTTPHEADER, ['Content-Type: application/x-www-form-urlencoded']);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $post_data);
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_ENCODING, 'gzip');
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
        $result = curl_exec($curl);
        curl_close( $curl );
        $result = json_decode($result, true);

        return $result;
    }



    public function getPaymentIdFromRequest(Request $request)
    {

        return (int) $_REQUEST['order_id'];
    }


    public function getCurrencyList()
    {
        return array("RUB","USD","EUR",
        );
    }



}
